@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,9 +19,11 @@
 #include <algorithm>
 #include <cstdio>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
+#include "segment.h"
 #include "ucs.h"
 #include "bitmap.h"
 #include "blob.h"
@@ -166,7 +168,8 @@ int Features::test_CEFIJLlT( const Charset & charset ) const
     int hdiff;
     if( b.bottom_hook( &hdiff ) )
       {
-      if( hdiff > b.height() / 2 && rp.increasing( rp.pos( 80 ) ) ) return 'J';
+      if( hdiff > b.height() / 2 && rp.increasing( rp.pos( 80 ), 1 ) &&
+          !rp.decreasing() ) return 'J';
       if( -hdiff > b.height() / 2 )
         { if( 5 * lp[lp.pos(80)] < 2 * b.width() ) return 'L'; else return 'v'; }
       }
@@ -207,7 +210,7 @@ int Features::test_CEFIJLlT( const Charset & charset ) const
           vbar(0).h_overlaps( hbar(1) ) )
         {
         if( hbars() == 3 &&
-            Ocrad::similar( hbar(0).width(), hbar(2).width(), 10 ) &&
+            Ocrad::similar( hbar(0).width(), hbar(2).width(), 10, 2 ) &&
             10 * hbar(2).width() >= 9 * hbar(1).width() &&
             hbar(0).left() <= hbar(1).left() + 1 )
           return 'E';
@@ -423,7 +426,8 @@ int Features::test_frst( const Rectangle & charbox ) const
           }
         }
       if( Ocrad::similar( b.height(), b.width(), 40 ) &&
-          vscan()[lp.pos(15)] == 3 && vscan()[lp.pos(85)] == 3 &&
+          segments_in_row( b.vpos( 15 ) ) == 3 &&
+          segments_in_row( b.vpos( 85 ) ) == 3 &&
           b.seek_right( row - 1, col ) < b.right() && lp.isctip() )
         return 'x';
       }
@@ -482,17 +486,18 @@ int Features::test_G() const
     if( row < b.bottom() && b.escape_right( row, col ) &&
         !b.escape_bottom( row, b.hcenter() ) )
       {
+      const int noise = std::max( 2, b.height() / 20 );
       int lrow, urow;
       for( lrow = row - 1 ; lrow > b.top(); --lrow )
         if( b.seek_right( lrow, b.hcenter() ) >= b.right() ) break;
       for( urow = lrow - 1 ; urow > b.top(); --urow )
         if( b.seek_right( urow, b.hcenter() ) < b.right() ) break;
-      lrow += 2; urow -= 1;
+      lrow += noise; urow -= noise;
       if( lrow < row && urow > b.top() )
         {
         int uwidth = b.seek_left( urow, b.right() ) - b.seek_right( urow, b.hcenter() );
         int lwidth = b.seek_left( lrow, b.right() ) - b.seek_right( lrow, b.hcenter() );
-        if( lrow - 1 <= b.vcenter() || lwidth > uwidth + 2 )
+        if( lrow - noise <= b.vcenter() || lwidth > uwidth + noise )
           return 'G';
         }
       }
@@ -522,7 +527,7 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
         {
         if( std::abs( hdiff ) <= b.height() / 8 )
           {
-          if( vscan()[b.height()/4] >= 3 ) return 'v';
+          if( segments_in_row( b.vpos( 30 ) ) >= 3 ) return 'v';
           if( bp.isconvex() )
             { if( 9 * wp[wp.pos(30)] > 10 * wp[wp.pos(50)] &&
                   9 * wp[wp.pos(50)] > 10 * wp[wp.pos(70)] ) return 'v';
@@ -536,11 +541,23 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
       const int rg2 = rp.max( rp.pos( 70 ), rp.pos( 90 ) );
       const int lc = ( lg + ( 2 * ( lp.limit() - rg ) ) ) / 3;
       const int lc2 = ( lg2 + lp.limit() - rg2 ) / 2;
-      if( bp.ispit() && 7 * tp.range() < 4 * b.height() &&
-          2 * wp[wp.pos(70)] < b.width() &&
-          ( ( Ocrad::similar( lg, rg, 20 ) &&
-            5 * wp[wp.pos(98)] + noise >= 4 * wp[wp.pos(70)] ) ||
-            ( lg > 1 && lg < rg && lc >= lc2 && !rp.increasing() ) ) ) return 'Y';
+      if( bp.ispit() && 7 * tp.range() < 4 * b.height() )
+        {
+        int row2 = b.top();
+        while( row2 < b.bottom() && segments_in_row( row2 ) != 2 ) ++row2;
+        int row1 = row2 + 1;
+        while( row1 < b.bottom() && segments_in_row( row1 ) != 1 ) ++row1;
+        if( row1 < b.bottom() ) row1 += wp[row1-b.top()] / 4;
+        if( row1 < b.bottom() && wp[row1-b.top()] < b.width() )
+          {
+          const int w1 = wp[row1-b.top()];
+          int row0 = w1 * ( row1 - row2 ) / ( b.width() - w1 ) + row1;
+          if( row0 < b.bottom() && 2 * wp[wp.pos(70)] < b.width() &&
+              ( Ocrad::similar( lg, rg, 20 ) ||
+              ( lg > 1 && lg < rg && lc >= lc2 && !rp.increasing() ) ) )
+            return 'Y';
+          }
+        }
       if( b.escape_top( b.vpos( 60 ), b.hcenter() ) && !lp.istip() )
         return 'u';
       if( lg < rg + 1 && !lp.increasing( lp.pos( 50 ) ) &&
@@ -563,16 +580,20 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
       if( Ocrad::similar( lg, rg, 80, 2 ) &&
           4 * lg < b.width() && 4 * rg < b.width() )
         {
-        if( lg > 1 && rg > 1 && lp.increasing() && rp.increasing() )
+        if( lg > 1 && rg > 1 && lp.increasing() && rp.increasing() &&
+            5 * tp[tp.pos(50)] > b.height() )
           return 'w';
-        if( hbars() == 1 && 5 * hbar(0).height() < b.height() &&
+        if( hbars() == 1 && 5 * ( hbar(0).height() - 1 ) < b.height() &&
             hbar(0).top() >= b.vpos( 30 ) && hbar(0).bottom() <= b.vpos( 60 ) &&
-            10 * hbar(0).width() > 9 * wp[hbar(0).vcenter()-b.top()] )
+            10 * hbar(0).width() > 9 * wp[hbar(0).vcenter()-b.top()] &&
+            Ocrad::similar( col_segment( hbar(0).vcenter(), hbar(0).hcenter() ).size(),
+                            hbar(0).height(), 30, 2 ) )
           {
           if( 9 * hbar(0).width() < 10 * wp[wp.pos(50)] ) return 'H';
           return 0;
           }
-        if( vscan()[rp.pos(60)] == 4 || vscan()[rp.pos(70)] == 4 )
+        if( segments_in_row( b.vpos( 60 ) ) == 4 ||
+            segments_in_row( b.vpos( 70 ) ) == 4 )
           {
           if( 2 * tp[tp.pos(50)] > b.height() ) return 'M';
           return 'w';
@@ -582,12 +603,15 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
             tp.minima( ( 2 * b.height() ) / 5 ) <= 2 && !lp.istpit() &&
             4 * std::abs( rp[rp.pos(20)] - rp[rp.pos(80)] ) <= b.width() )
           {
-          Rectangle r( b.left(), b.top(), b.hcenter(), b.bottom() );
-          Bitmap bm( b, r );
-          int hdiff;
-          if( bm.seek_bottom( bm.top(), bm.right() ) > bm.vcenter() &&
-              bm.bottom_hook( &hdiff ) && -2 * hdiff > bm.height() ) return 'u';
-          else return 'N';
+          const int row = b.top() + tp[tp.pos(50)];
+          if( row > b.vcenter() )
+            {
+            Rectangle r( b.left(), b.top(), b.hcenter(), b.bottom() );
+            Bitmap bm( b, r );
+            int hdiff;
+            if( bm.bottom_hook( &hdiff ) && -2 * hdiff > bm.height() ) return 'u';
+            }
+          if( row > b.vpos( 10 ) || vbars() >= 2 ) return 'N';
           }
         return 0;
         }
@@ -596,7 +620,9 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
       return 0;
       }
     if( bp.minima() <= 2 && 2 * b.width() > 5 * b.height() ) return '~';
-    if( bp.minima() == 3 ) return 'M';
+    if( bp.minima() == 3 &&
+        ( hbars() == 0 || ( hbars() == 1 && hbar(0).top() >= b.vpos( 20 ) ) ) )
+      return 'M';
     }
   return 0;
   }
@@ -642,7 +668,9 @@ int Features::test_hknwx( const Rectangle & charbox ) const
       bp.minima( urow + 1 ) == 3 )
     return 'w';
   if( urow <= b.vpos( 20 ) && tp.minima( b.height() / 4 ) == 1 &&
-      ( b.height() < b.width() || Ocrad::similar( b.height(), b.width(), 40 ) ) )
+      Ocrad::similar( b.height(), b.width(), 40 ) &&
+      ( 8 * ( rp[rp.pos(50)] - 1 ) <= b.width() ||
+        tp[tp.pos(100)] > b.height() / 2 ) )
     return 'n';
   return 0;
   }
