@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,9 +18,11 @@
 
 #include <algorithm>
 #include <cctype>
+#include <climits>
 #include <cstdio>
 #include <string>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rational.h"
@@ -39,7 +41,7 @@ namespace {
 // Return the character position >= first preceding a big gap or eol.
 //
 int find_big_gap( const Textline & line, const int first,
-                  const int space_width_limit ) throw()
+                  const int space_width_limit )
   {
   int i = first;
   while( i + 1 < line.characters() )
@@ -59,7 +61,7 @@ Textline::Textline( const Textline & tl )
   : Track( tl ), big_initials_( tl.big_initials_ )
   {
   cpv.reserve( tl.cpv.size() );
-  for( unsigned int i = 0; i < tl.cpv.size(); ++i )
+  for( unsigned i = 0; i < tl.cpv.size(); ++i )
     cpv.push_back( new Character( *tl.cpv[i] ) );
   }
 
@@ -70,32 +72,32 @@ Textline & Textline::operator=( const Textline & tl )
     {
     Track::operator=( tl );
     big_initials_ = tl.big_initials_;
-    for( unsigned int i = 0; i < cpv.size(); ++i ) delete cpv[i];
+    for( unsigned i = 0; i < cpv.size(); ++i ) delete cpv[i];
     cpv.clear();
     cpv.reserve( tl.cpv.size() );
-    for( unsigned int i = 0; i < tl.cpv.size(); ++i )
+    for( unsigned i = 0; i < tl.cpv.size(); ++i )
       cpv.push_back( new Character( *tl.cpv[i] ) );
     }
   return *this;
   }
 
 
-Textline::~Textline() throw()
+Textline::~Textline()
   {
-  for( unsigned int i = 0; i < cpv.size(); ++i ) delete cpv[i];
+  for( unsigned i = 0; i < cpv.size(); ++i ) delete cpv[i];
   }
 
 
 void Textline::set_track()
   {
   std::vector< Rectangle > rv;
-  for( unsigned int i = big_initials_; i < cpv.size(); ++i )
+  for( unsigned i = big_initials_; i < cpv.size(); ++i )
     if( !cpv[i]->maybe(' ') ) rv.push_back( *cpv[i] );
   Track::set_track( rv );
   }
 
 
-void Textline::verify_big_initials() throw()
+void Textline::verify_big_initials()
   {
   while( big_initials_ > 0 &&
          cpv[big_initials_-1]->height() <= 2 * mean_height() )
@@ -103,7 +105,7 @@ void Textline::verify_big_initials() throw()
   }
 
 
-Character & Textline::character( const int i ) const throw()
+Character & Textline::character( const int i ) const
   {
   if( i < 0 || i >= characters() )
     Ocrad::internal_error( "character, index out of bounds" );
@@ -111,7 +113,7 @@ Character & Textline::character( const int i ) const throw()
   }
 
 
-Character * Textline::character_at( const int col ) const throw()
+Character * Textline::character_at( const int col ) const
   {
   for( int i = 0; i < characters(); ++i )
     if( cpv[i]->h_includes( col ) ) return cpv[i];
@@ -119,13 +121,13 @@ Character * Textline::character_at( const int col ) const throw()
   }
 
 
-Rectangle Textline::charbox( const Character & c ) const throw()
+Rectangle Textline::charbox( const Character & c ) const
   {
   return Rectangle( c.left(), top( c.hcenter() ), c.right(), bottom( c.hcenter() ) );
   }
 
 
-void Textline::delete_character( const int i ) throw()
+void Textline::delete_character( const int i )
   {
   if( i < 0 || i >= characters() )
     Ocrad::internal_error( "delete_character, index out of bounds" );
@@ -221,7 +223,7 @@ void Textline::join( Textline & tl )
   }
 
 
-int Textline::mean_height() const throw()
+int Textline::mean_height() const
   {
   int c = 0, sum = 0;
 
@@ -232,7 +234,7 @@ int Textline::mean_height() const throw()
   }
 
 
-Rational Textline::mean_width() const throw()
+Rational Textline::mean_width() const
   {
   int c = 0, sum = 0;
 
@@ -244,7 +246,7 @@ Rational Textline::mean_width() const throw()
   }
 
 
-Rational Textline::mean_gap_width( const int first, int last ) const throw()
+Rational Textline::mean_gap_width( const int first, int last ) const
   {
   if( last < 0 ) last = characters() - 1;
   int sum = 0;
@@ -257,7 +259,7 @@ Rational Textline::mean_gap_width( const int first, int last ) const throw()
   }
 
 
-int Textline::mean_hcenter() const throw()
+int Textline::mean_hcenter() const
   {
   int c = 0, sum = 0;
 
@@ -268,7 +270,7 @@ int Textline::mean_hcenter() const throw()
   }
 
 
-int Textline::mean_vcenter() const throw()
+int Textline::mean_vcenter() const
   {
   int c = 0, sum = 0;
 
@@ -279,7 +281,7 @@ int Textline::mean_vcenter() const throw()
   }
 
 
-void Textline::print( const Control & control ) const throw()
+void Textline::print( const Control & control ) const
   {
   for( int i = 0; i < characters(); ++i ) character( i ).print( control );
   std::fputs( "\n", control.outfile );
@@ -287,7 +289,7 @@ void Textline::print( const Control & control ) const throw()
 
 
 void Textline::dprint( const Control & control, const bool graph,
-                       const bool recursive ) const throw()
+                       const bool recursive ) const
   {
   if( graph || recursive )
     std::fprintf( control.outfile, "mean height = %d, track segments = %d\n",
@@ -303,14 +305,14 @@ void Textline::dprint( const Control & control, const bool graph,
   }
 
 
-void Textline::xprint( const Control & control ) const throw()
+void Textline::xprint( const Control & control ) const
   {
   for( int i = 0; i < characters(); ++i )
     character( i ).xprint( control );
   }
 
 
-void Textline::cmark( Page_image & page_image ) const throw()
+void Textline::cmark( Page_image & page_image ) const
   {
   for( int i = 0; i < characters(); ++i )
     page_image.draw_rectangle( character( i ) );
