@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,9 +20,11 @@
 #include <cstdio>
 #include <string>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
+#include "segment.h"
 #include "mask.h"
 #include "track.h"
 #include "bitmap.h"
@@ -39,7 +41,6 @@ namespace {
 struct Zone
   {
   Mask mask;
-//  std::vector< std::vector< Blob * > > blobp_matrix;
   std::vector< Blob * > blobp_vector;
   Zone( const Rectangle & re ) : mask( re ) {}
   void join( Zone & z );
@@ -55,41 +56,32 @@ void Zone::join( Zone & z )
   }
 
 
-int blobs_in_page( const std::vector< Zone > & zone_vector ) throw()
+int blobs_in_page( const std::vector< Zone > & zone_vector )
   {
   int sum = 0;
-  for( unsigned int i = 0; i < zone_vector.size(); ++i )
+  for( unsigned i = 0; i < zone_vector.size(); ++i )
     sum += zone_vector[i].blobp_vector.size();
   return sum;
   }
 
 
-void bprint( const std::vector< Zone > & zone_vector, FILE * const outfile ) throw()
+void bprint( const std::vector< Zone > & zone_vector, FILE * const outfile )
   {
 //  std::fprintf( outfile, "page size %dw x %dh\n", width(), height() );
   std::fprintf( outfile, "total zones in page %d\n", (int)zone_vector.size() );
   std::fprintf( outfile, "total blobs in page %d\n\n", blobs_in_page( zone_vector ) );
-  for( unsigned int zindex = 0; zindex < zone_vector.size(); ++zindex )
+  for( unsigned zindex = 0; zindex < zone_vector.size(); ++zindex )
     {
     const Rectangle & r = zone_vector[zindex].mask;
-//    const std::vector< std::vector< Blob * > > & blobp_matrix = zone_vector[zindex].blobp_matrix;
     const std::vector< Blob * > & blobp_vector = zone_vector[zindex].blobp_vector;
 
     std::fprintf( outfile, "zone %d of %d\n", zindex + 1, (int)zone_vector.size() );
     std::fprintf( outfile, "zone size %dw x %dh\n", r.width(), r.height() );
-//    std::fprintf( outfile, "total cuts in zone %d\n", (int)blobp_matrix.size() );
-//    std::fprintf( outfile, "total blobs in zone %d\n\n", blobs_in_zone( zone_vector, zindex ) );
-    std::fprintf( outfile, "total blobs in zone %d\n\n",
-                  (int)zone_vector[zindex].blobp_vector.size() );
+    std::fprintf( outfile, "total blobs in zone %u\n\n",
+                  (unsigned)zone_vector[zindex].blobp_vector.size() );
 
-//    for( unsigned int cut = 0; cut < blobp_matrix.size(); ++cut )
-//      {
-//      std::fprintf( outfile, "cut %d blobs %d\n", cut + 1, (int)blobp_matrix[cut].size() );
-//      for( unsigned int i = 0; i < blobp_matrix[cut].size(); ++i )
-//        blobp_matrix[cut][i]->print( outfile );
-      for( unsigned int i = 0; i < blobp_vector.size(); ++i )
-        blobp_vector[i]->print( outfile );
-//      }
+    for( unsigned i = 0; i < blobp_vector.size(); ++i )
+      blobp_vector[i]->print( outfile );
     }
   }
 
@@ -101,8 +93,8 @@ inline void join_blobs( std::vector< Blob * > & blobp_vector,
   {
   if( p1->top() > p2->top() )
     {
-    Blob * temp = p1; p1 = p2; p2 = temp;
-    std::replace( v2.begin(), v2.begin() + i + 1, p2, p1 );
+    Blob * const temp = p1; p1 = p2; p2 = temp;
+    std::replace( v2.begin(), v2.begin() + ( i + 1 ), p2, p1 );
     }
   else std::replace( v1.begin() + i, v1.end(), p2, p1 );
 
@@ -116,24 +108,26 @@ inline void join_blobs( std::vector< Blob * > & blobp_vector,
   }
 
 
-void ignore_abnormal_blobs( std::vector< Blob * > & blobp_vector ) throw()
+void ignore_abnormal_blobs( std::vector< Blob * > & blobp_vector )
   {
   for( int i = blobp_vector.size() - 1; i >= 0; --i )
     {
     Blob & b = *blobp_vector[i];
     if( b.height() > 35 * b.width() || b.width() > 25 * b.height() )
-      { delete blobp_vector[i];
-      blobp_vector.erase( blobp_vector.begin() + i ); }
+      {
+      delete blobp_vector[i];
+      blobp_vector.erase( blobp_vector.begin() + i );
+      }
     }
   }
 
 
-void ignore_small_blobs( std::vector< Blob * > & blobp_vector ) throw()
+void ignore_small_blobs( std::vector< Blob * > & blobp_vector )
   {
   int to = 0, blobs = blobp_vector.size();
   for( int from = 0; from < blobs; ++from )
     {
-    Blob * p = blobp_vector[from];
+    Blob * const p = blobp_vector[from];
     if( p->height() > 4 || p->width() > 4 ||
         ( ( p->height() > 2 || p->width() > 2 ) && p->area() > 5 ) )
       { blobp_vector[from] = blobp_vector[to]; blobp_vector[to] = p; ++to; }
@@ -146,7 +140,7 @@ void ignore_small_blobs( std::vector< Blob * > & blobp_vector ) throw()
   }
 
 
-void remove_top_bottom_noise( std::vector< Blob * > & blobp_vector ) throw()
+void remove_top_bottom_noise( std::vector< Blob * > & blobp_vector )
   {
   int blobs = blobp_vector.size();
   for( int i = 0; i < blobs; ++i )
@@ -167,7 +161,7 @@ void remove_top_bottom_noise( std::vector< Blob * > & blobp_vector ) throw()
   }
 
 
-void remove_left_right_noise( std::vector< Blob * > & blobp_vector ) throw()
+void remove_left_right_noise( std::vector< Blob * > & blobp_vector )
   {
   int blobs = blobp_vector.size();
   for( int i = 0; i < blobs; ++i )
@@ -190,37 +184,35 @@ void remove_left_right_noise( std::vector< Blob * > & blobp_vector ) throw()
 
 void find_holes( std::vector< Zone > & zone_vector )
   {
-  for( unsigned int zi = 0; zi < zone_vector.size(); ++zi )
+  for( unsigned zi = 0; zi < zone_vector.size(); ++zi )
     {
-//    std::vector< std::vector< Blob * > > & blobp_matrix = zone_vector[zi].blobp_matrix;
     std::vector< Blob * > & blobp_vector = zone_vector[zi].blobp_vector;
-//    for( unsigned int bmi = 0; bmi < blobp_matrix.size(); ++bmi )
-//      {
-//      std::vector< Blob * > & blobp_vector = blobp_matrix[bmi];
-      for( unsigned int bvi = 0; bvi < blobp_vector.size(); ++bvi )
-        blobp_vector[bvi]->find_holes();
-//      }
+    for( unsigned bvi = 0; bvi < blobp_vector.size(); ++bvi )
+      blobp_vector[bvi]->find_holes();
     }
   }
 
 
 void ignore_wide_blobs( const Rectangle & re,
-                        std::vector< Blob * > & blobp_vector ) throw()
+                        std::vector< Blob * > & blobp_vector )
   {
-  for( unsigned int i = 0; i < blobp_vector.size(); )
+  for( unsigned i = 0; i < blobp_vector.size(); )
     {
     Blob & b = *blobp_vector[i];
     if( 2 * b.width() < re.width() ) { ++i; continue; }
     blobp_vector.erase( blobp_vector.begin() + i );
     if( 4 * b.area() <= 3 * b.size() )
       {
-      unsigned int blobs;
-      for( blobs = i; blobs < blobp_vector.size(); ++blobs )
-        if( blobp_vector[blobs]->top() > b.bottom() ) break;
-      blobs -= i;
-      if( blobs <= b.size() / 400U )
+      int blobs = 0;
+      for( unsigned j = i; j < blobp_vector.size(); ++j )
         {
-        if( 4 * b.area() <= b.size() ) { delete &b; continue; }
+        if( blobp_vector[j]->top() > b.bottom() ) break;
+        if( blobp_vector[j]->size() >= 16 ) ++blobs;
+        }
+      if( blobs <= b.size() / 400 )
+        {
+        if( 4 * b.area() <= b.size() )		// thin grid or frame
+          { delete &b; continue; }
         b.find_holes();
         bool frame = false;
         if( b.holes() < std::min( b.height(), b.width() ) )
@@ -233,49 +225,48 @@ void ignore_wide_blobs( const Rectangle & re,
         if( frame ) { delete &b; continue; }
         }
       }
-    // image, not frame
+    // picture, not frame
     if( 5 * b.width() > 4 * re.width() && 5 * b.height() > 4 * re.height() )
       {
-      for( unsigned int j = 0; j < blobp_vector.size(); ++j )
+      for( unsigned j = 0; j < blobp_vector.size(); ++j )
         delete blobp_vector[j];
       blobp_vector.clear(); delete &b; break;
       }
-    for( int j = blobp_vector.size(); --j >= 0; )
+    for( unsigned j = blobp_vector.size(); j > i; )
       {
-      const Blob & b2 = *blobp_vector[j];
+      const Blob & b2 = *blobp_vector[--j];
       if( b.includes( b2 ) )
         { delete &b2; blobp_vector.erase( blobp_vector.begin() + j ); }
-      else if( b2.top() < b.top() ) break;
       }
     delete &b;
     }
   }
 
 
-int mean_blob_height( const std::vector< Blob * > & blobp_vector ) throw()
+int mean_blob_height( const std::vector< Blob * > & blobp_vector )
   {
   int mean_height = 0;
-  unsigned int samples = 0;
+  unsigned samples = 0;
   std::vector< int > height_distrib;
 
-  for( unsigned int i = 0; i < blobp_vector.size(); ++i )
+  for( unsigned i = 0; i < blobp_vector.size(); ++i )
     {
-    const unsigned int h = blobp_vector[i]->height();
-    const unsigned int w = blobp_vector[i]->width();
+    const unsigned h = blobp_vector[i]->height();
+    const unsigned w = blobp_vector[i]->width();
     if( h < 10 || w >= 3 * h ) continue;
     if( h >= height_distrib.size() ) height_distrib.resize( h + 1 );
     ++height_distrib[h]; ++samples;
     }
   if( !height_distrib.size() )
-    for( unsigned int i = 0; i < blobp_vector.size(); ++i )
+    for( unsigned i = 0; i < blobp_vector.size(); ++i )
       {
-      const unsigned int h = blobp_vector[i]->height();
+      const unsigned h = blobp_vector[i]->height();
       if( h >= height_distrib.size() ) height_distrib.resize( h + 1 );
       ++height_distrib[h]; ++samples;
       }
 
   int valid_samples = 0;
-  for( unsigned int i = 0, count = 0; i < height_distrib.size(); ++i )
+  for( unsigned i = 0, count = 0; i < height_distrib.size(); ++i )
     {
     const int a = height_distrib[i];
     if( 10 * ( count + a ) >= samples && 10 * count < 9 * samples )
@@ -287,19 +278,19 @@ int mean_blob_height( const std::vector< Blob * > & blobp_vector ) throw()
   }
 
 
-int analyse_layout( const std::vector< Blob * > & blobp_vector,
+int analyse_layout( std::vector< Blob * > & blobp_vector,
                     std::vector< Zone > & zone_vector )
   {
   if( !blobp_vector.size() ) return 0;
   const int mean_height = mean_blob_height( blobp_vector );
   zone_vector.push_back( Zone( *blobp_vector[0] ) );
   zone_vector.back().blobp_vector.push_back( blobp_vector[0] );
-  for( unsigned int i = 1; i < blobp_vector.size(); ++i )
+  for( unsigned i = 1; i < blobp_vector.size(); ++i )
     {
     Blob & b = *blobp_vector[i];
-    if( b.height() > 10 * mean_height ) continue;
+    if( b.height() > 10 * mean_height ) { delete &b; continue; }
     int first = -1;
-    for( unsigned int j = 0; j < zone_vector.size(); ++j )
+    for( unsigned j = 0; j < zone_vector.size(); ++j )
       {
       if( zone_vector[j].mask.distance( b ) < 2 * mean_height )
         {
@@ -322,43 +313,44 @@ int analyse_layout( const std::vector< Blob * > & blobp_vector,
       zone_vector.back().blobp_vector.push_back( &b );
       }
     }
+  blobp_vector.clear();
 
   // sort zone_vector
   int botmax = ( zone_vector.size() ? zone_vector[0].mask.bottom() : 0 );
   std::vector< int > cut_index_vector;
-  for( unsigned int i = 1; i < zone_vector.size(); ++i )
+  for( unsigned i = 1; i < zone_vector.size(); ++i )
     {
     if( zone_vector[i].mask.top() > botmax ) cut_index_vector.push_back( i );
     botmax = std::max( botmax, zone_vector[i].mask.bottom() );
     }
   cut_index_vector.push_back( zone_vector.size() );
-  for( unsigned int begin = 0, cut = 0; cut < cut_index_vector.size(); ++cut )
+  for( unsigned begin = 0, cut = 0; cut < cut_index_vector.size(); ++cut )
     {
-    const unsigned int end = cut_index_vector[cut];
-    for( unsigned int i = begin; i + 1 < end; ++i )
+    const unsigned end = cut_index_vector[cut];
+    for( unsigned i = begin; i + 1 < end; ++i )
       {
-      unsigned int first = i;
-      for( unsigned int j = i + 1; j < end; ++j )
+      unsigned first = i;
+      for( unsigned j = i + 1; j < end; ++j )
         if( zone_vector[j].mask.precedes( zone_vector[first].mask ) )
           first = j;
       if( first != i ) std::swap( zone_vector[i], zone_vector[first] );
       }
     bool join = ( end - begin > 1 );
-    for( unsigned int i = begin; join && i < end; ++i )
+    for( unsigned i = begin; join && i < end; ++i )
       if( zone_vector[i].blobp_vector.size() > 80 ||
           zone_vector[i].mask.v_distance( zone_vector[begin].mask ) >
           zone_vector[i].mask.height() + zone_vector[begin].mask.height() )
         join = false;
-    for( unsigned int i = begin; join && i < end; ++i )
+    for( unsigned i = begin; join && i < end; ++i )
       if( zone_vector[i].mask.height() > 4 * mean_blob_height( zone_vector[i].blobp_vector ) )
         join = false;
     if( join )
       {
-      for( unsigned int i = begin + 1; i < end; ++i )
+      for( unsigned i = begin + 1; i < end; ++i )
         zone_vector[begin].join( zone_vector[i] );
-      zone_vector.erase( zone_vector.begin() + begin + 1,
+      zone_vector.erase( zone_vector.begin() + ( begin + 1 ),
                          zone_vector.begin() + end );
-      for( unsigned int i = cut; i < cut_index_vector.size(); ++i )
+      for( unsigned i = cut; i < cut_index_vector.size(); ++i )
         cut_index_vector[i] -= ( end - begin - 1 );
       ++begin;
       }
@@ -384,8 +376,8 @@ void scan_page( const Page_image & page_image, std::vector< Zone > & zone_vector
       {
       const int dcol = col - re.left();
       if( !page_image.get_bit( row, col, zthreshold ) )
-        new_data[dcol] = 0;			// white point
-      else					// black point
+        new_data[dcol] = 0;			// white pixel
+      else					// black pixel
         {
         Blob *p;
         Blob *lp  = ( (dcol > 0) ? new_data[dcol-1] : 0 );
@@ -409,7 +401,7 @@ void scan_page( const Page_image & page_image, std::vector< Zone > & zone_vector
       }
     }
 
-  if( debug_level <= 99 && (int)blobp_vector.size() > 3 )
+  if( debug_level <= 99 && blobp_vector.size() > 3 )
     {
     ignore_wide_blobs( re, blobp_vector );
     ignore_small_blobs( blobp_vector );
@@ -419,11 +411,11 @@ void scan_page( const Page_image & page_image, std::vector< Zone > & zone_vector
     }
 
   if( layout && re.width() > 200 && re.height() > 200 &&
-      (int)blobp_vector.size() > 3 )
+      blobp_vector.size() > 3 )
     {
     analyse_layout( blobp_vector, zone_vector );
     if( debug_level <= 99 && zone_vector.size() > 1 )
-      for( unsigned int i = 0; i < zone_vector.size(); ++i )
+      for( unsigned i = 0; i < zone_vector.size(); ++i )
         ignore_wide_blobs( zone_vector[i].mask, zone_vector[i].blobp_vector );
     }
   else
@@ -457,11 +449,11 @@ Textpage::Textpage( const Page_image & page_image, const char * const filename,
   if( debug_level > 95 || ( debug_level > 89 && debug_level < 94 ) ) return;
 
   // build a Textblock for every zone with text
-  for( unsigned int i = 0; i < zone_vector.size(); ++i )
+  for( unsigned i = 0; i < zone_vector.size(); ++i )
     {
-    Textblock * tbp = new Textblock( page_image, zone_vector[i].mask,
-                                     zone_vector[i].blobp_vector );
-    if( debug_level < 90 )
+    Textblock * const tbp = new Textblock( page_image, zone_vector[i].mask,
+                                           zone_vector[i].blobp_vector );
+    if( tbp->textlines() && debug_level < 90 )
       tbp->recognize( control.charset, control.filter );
     if( tbp->textlines() ) tbpv.push_back( tbp );
     else delete tbp;
@@ -481,7 +473,7 @@ Textpage::Textpage( const Page_image & page_image, const char * const filename,
     {
     Page_image tmp( page_image );
     if( ( debug_level - 70 ) & 1 )	// mark zones
-      for( unsigned int i = 0; i < zone_vector.size(); ++i )
+      for( unsigned i = 0; i < zone_vector.size(); ++i )
         {
         if( debug_level == 71 ) tmp.draw_mask( zone_vector[i].mask );
         else tmp.draw_rectangle( zone_vector[i].mask );
@@ -500,13 +492,13 @@ Textpage::Textpage( const Page_image & page_image, const char * const filename,
   }
 
 
-Textpage::~Textpage() throw()
+Textpage::~Textpage()
   {
   for( int i = textblocks() - 1; i >= 0; --i ) delete tbpv[i];
   }
 
 
-const Textblock & Textpage::textblock( int i ) const throw()
+const Textblock & Textpage::textblock( const int i ) const
   {
   if( i < 0 || i >= textblocks() )
     Ocrad::internal_error( "Textpage::textblock, index out of bounds" );
@@ -514,7 +506,7 @@ const Textblock & Textpage::textblock( int i ) const throw()
   }
 
 
-int Textpage::textlines() const throw()
+int Textpage::textlines() const
   {
   int total = 0;
   for( int i = 0; i < textblocks(); ++i )
@@ -523,7 +515,7 @@ int Textpage::textlines() const throw()
   }
 
 
-int Textpage::characters() const throw()
+int Textpage::characters() const
   {
   int total = 0;
   for( int i = 0; i < textblocks(); ++i )
@@ -532,7 +524,7 @@ int Textpage::characters() const throw()
   }
 
 
-void Textpage::print( const Control & control ) const throw()
+void Textpage::print( const Control & control ) const
   {
   if( control.outfile )
     for( int i = 0; i < textblocks(); ++i )
@@ -540,7 +532,7 @@ void Textpage::print( const Control & control ) const throw()
   }
 
 
-void Textpage::xprint( const Control & control ) const throw()
+void Textpage::xprint( const Control & control ) const
   {
   if( !control.exportfile ) return;
 
