@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,16 +23,19 @@ class Textpage : public Rectangle
   const std::string name;
   std::vector< Textblock * > tbpv;
 
+  Textpage( const Textpage & );			// declared as private
+  void operator=( const Textpage & );		// declared as private
+
 public:
   Textpage( const Page_image & page_image, const char * const filename,
             const Control & control, const bool layout );
-  ~Textpage() throw();
+  ~Textpage();
 
-  const Textblock & textblock( int i ) const throw();
-  int textblocks() const throw() { return tbpv.size(); }
-  int textlines() const throw();
-  int characters() const throw();
+  const Textblock & textblock( const int i ) const;
+  int textblocks() const { return tbpv.size(); }
+  int textlines() const;
+  int characters() const;
 
-  void print( const Control & control ) const throw();
-  void xprint( const Control & control ) const throw();
+  void print( const Control & control ) const;
+  void xprint( const Control & control ) const;
   };
